\ GoertzelModeB.fth Taqoz V2.8
\ uses code from https://forums.parallax.com/discussion/174531/taqoz-reloaded-2-8-better-readability-named-local-variables#latest
\ CWE 30.10.2022

!polls \ clear multitasking

IFDEF *GoertzelM*	
    1 cogstop
    oldorg org
    FORGET *GoertzelM*  }

pub *GoertzelM*    PRINT" Goertzel Mode GoertzelModeB.fth" ;

0 bytes oldorg

44 value dacPin 
46 value compPin
46 value adcPin

long goeResult
long goeCount


$0100 value sinTab#
$0040 value tablen#
2147483647 tablen# 2 / / value deltaW#  

40 ( kHz ) 65536 * 32768 200000 */ value ddsFreq# \ kHz f. 1 MHz: 65536 32768 200 */ 

100 value cycles#
%1111_0001_0_101_1111 16 << 
cycles# + value ddsD#
\ page 31 
\ 1111 dddd 0ppp p111: 
\ dddd=0001 output X0 on all 4 dac channels
\ pppp= input pin group pppp=1010 = 44/16 - 1

%0000_0100_011 9 << \ %x0xx_x1xx	Base pin +2 is summed; 011 for length 64
\ %0000_0100_010 9 << \ %x0xx_x1xx	Base pin +2 is summed; 010 for length 128
sinTab# + value ddsS# 
\ https://docs.google.com/document/d/1gn6oaT5Ib7CytvlZHacmrSbVBJsD9t_-kmvjd7nUR6o/edit# page36


\ DAC and Comparator input Modes ####################################################################

{
Text from extend.fth
DAC Output Modes
DIR enables output, OUT enables ADC
%0000_0000_000_1010000000000_00_00000_0		P_DAC_990R_3V			DAC 990Ω, 3.3V peak, ADC 1x → IN
%xxxx_xxxx_xxx_xxxxxDDDDDDDD_xx_xxxxx_x
DDDDDDDD = 8-bit DAC value
}


: dac8 ( value ) \ set mode 8 bit dac and output value 255=3.3V
   dacPin pin
   ( value ) 8<< 
   %0000_0000_000_1010000000000_00_00000_0 \ DAC 990Ω, 3.3V peak, ADC 1x → IN
   \ %0000_0000_000_1010100000000_00_00000_0		\ DAC 600Ω, 2.0V peak, ADC 1x → IN
   or wrpin
   L \ set pin low ==> enables output
;

: dacTest
   key drop
   begin
      255 for
         \ i . 
         i dac8
      next
   key until
;

: comp8 ( threshold -- true/false ) \ 8 bit comparator
   compPin pin
   8<< 
   %0000_0000_000_11000_00000000_00_00000_0 \ Level comparator ( with 1k5 output if enabled )
   or wrpin
   F \ set pin to float ==> disables output
   R \ read pin state as input
;

: compTest {: comp# } \ dacPin and compPin linked with wire
      255 for
         i dac8 \ output a voltage
         comp# comp8 . \ compare at compPin 
      next
;

\ Goertzel Mode ######################################################################
\ 


: sinLut.old \ write table into LUT sin:cos:sin:cos
   tablen# for
      10 0   i deltaW# * QROTATE \ 127
      \ crlf i . . . 
      24 <<
      swap 16 <<
      or
      dup 16 >> \ lower 16 bits
      or
      i sinTab# + LUT!
   next ;

: sinLut \ write table into LUT sin:cos:cos:sin
   tablen# for
      10 0   i deltaW# * QROTATE \ higher 16 bits with low amplitude
      \ crlf i . . . 
      24 <<
      swap 16 << or
      
      127 0   i deltaW# * QROTATE \ lower 16 bits
      8 <<
      or
      
      or
      i sinTab# + LUT!
   next ;

code setDdsFreq '( freq )
      setxfrq a 
      jmp #@DROP
end

: setDdsDac
   dacPin pin 
   \ %AAAA_BBBB_FFF_MMMMMMMMMMMMM_TT_SSSSS_0
   cogid 8<< \ source for DAC output
   \ %0000_0000_000_1010000000000_00_00000_0 or \ DAC 990Ω, 3.3V peak, ADC 1x → IN
   %0000_0000_000_1011000000000_00_00000_0 or \ DAC 124Ohm, 3.3V 
   %0000_0000_000_0000000000000_01_00000_0 or \ p_channel
   wrpin
   L \ set pin low ==> enables output
;

: setGoeAdc
   adcPin pin
   \ %100011_0000000_00_00000_0 wrpin \ Gain 1
   %100100_0000000_00_00000_0 wrpin \ Gain 3.16
   \ %100101_0000000_00_00000_0 wrpin \ Gain 10
;

code goeAsm '( ddsS  ddsD -- angle length ) for first and single
   xcont a,b ' prerun
   xcont a,b ' start
   getxacc r2 ' to clear the accumulators: read cos, prepare s-field
   mov     r2,0 ' read sine
   xcont a,b ' wait and get result
   getxacc a ' read cos, prepare s-field
   mov     b,0 ' read sine
   qvector a,b
   getqx   a
   getqy   b
   ret
end


code nextGoeAsm '( ddsS  ddsD -- angle length ) for following
   xcont a,b ' wait and get result
   getxacc a ' read cos, prepare s-field
   mov     b,0 ' read sine
   qvector a,b
   getqx   a
   getqy   b
   ret
end


: doGoe ( -- length angle ) \ set up and do Goertzel once
   sinLut \ fill LUT Table
   ddsFreq# setDdsFreq
   setDdsDac
   setGoeAdc
   ddsS# ddsD# goeAsm
;

: doNextGoe ( -- length angle )
   ddsS# ddsD# nextGoeAsm
;


: testGoeSingle
   key drop
   begin
      crlf 
      0
      4 for \ average
         doGoe swap drop
         +
      next 
      4 /
      cycles# / . \ 360 $7fff_ffff */ . 
      1000 ms 
   key until
;

: testNextGoe
   key drop
   doGoe 2drop \ set up
   begin
      crlf 
      0
      4 for \ average
         doNextGoe swap drop
         +
      next 
      4 /
      cycles# / . \ 360 $7fff_ffff */ . 
      1000 ms 
   key until
;

: goeCog \ to be run in a second cog, will update goeResult
   doGoe 2drop \ set up
   0 goeCount !
   begin
      0
      4 for \ average
         doNextGoe swap drop
         +
      next 
      4 /
      cycles# / goeResult ! 
      goeCount ++
      \ 15 ms
   again
;

' goeCog 1 run \ run in cog1

: testGoeCog
   key drop
   begin
      crlf 
      goeCount @ .
      goeResult @ .
      1000 ms 
   key until
;

\ Application Touch sensor: 1MHz, R=22k between Output and input, Adc_gain=1 gives high number 360. Input touched gives about <140.

\ Application reflective infrared light sensor TCRT5000: 
\ Direct output from 124Ohms dac to ir emitter diode. 
\ receiver transistor collector via 2k2 to 3.3V and C= 0.1uF coupling to input.
\ receiver emitter to GND
\ adc gain 3.16





